;
; On-board flash programmer on MachZ BUR environment for AMD 8-bit chips
;
; This programmer writes variable-length payload into flash device.
;
; Programmer takes one parameter record at ZTAG manager software as
; starting address in flash.
;
; Programmer will program entire payload to flash chip, starting from defined
; address, without checking the overruns. If the chip is smaller than needed, wrap-around
; will occure.
;
; Programmer is supporting AM29F0xx (8-bit) family with devices JEDEC 
; algorithm and 64K uniform sectors.
;
; (c)2000 ZF Linux Devices, Inc.
;
; Target Chip:		AMD AM29F0xx 8-bit devices JEDEC algorithm and 64K uniform sectors
; Size:			
; Chipselect:		ms_cs0
; Mode:			8-bit
; Chip page:		defined by Parameter record
; Prg Window base:	E0000h
; Window size:		10000h (64K)
;
; This code executes as BUR "Load and execute" function. It will fetch parameter
; record and payload code following to the executable code in dongle.
;
; Compile using MASM 6.11 with following command line:
;
;	ml /AT /Zm am29fxxx.asm
;
;-------------------------------------------------------------------------------
; LJ 9/15/2006
;
; ZTWA (ZTAG Workaround) option:
; The ZTAG 'accelerator' on certain ZFx86 chips may not be reliable. To get
; around this problem, build this flash programmer with the ZTWA option:
;
;	ml /AT /Zm /DZTWA am29fxxx.asm
;
; When built this way, the flash programmer will not rely on the BUR routines,
; which do accelerated ZTAG reads. Non-accelerated ZTAG reads are about 6 times
; slower. Also, this program expects the dongle image to contain the flash
; program start address (command FE), and the flash ROM image (command F0), and
; nothing more. This flash programmer should be loaded via the serial port,
; with the ZFix ymodem download command, 'yload':
;
; yload 70:0
;
; (start YMODEM transfer from communications program)
;
; g 70:0
;
; The flash programmer will now program the AMD flash ROM according to the
; dongle's start address and ROM image commands.
;
;-------------------------------------------------------------------------------
; The resulting .COM file can be directly used as payload for 
; BUR "Upload and Execute" function
;

.MODEL TINY
.CODE
.486p

PRGBASE		=	0E0000h			; where to create programming window

		org	0
	
START:		jmp	around

		include FLASHPGM.MAC

around:		push	cs
		pop	es
		
		call	CRLF
ifdef ZTWA
		SAYCRLF	'AMD AM29F0xx Series 64K Uniform Sectors JEDEC Flash Programmer V1.09 ZTWA'
		SAYCRLF	'-------------------------------------------------------------------------'		
else
		SAYCRLF	'AMD AM29F0xx Series 64K Uniform Sectors JEDEC Flash Programmer V1.09'
		SAYCRLF	'------------------------------------------------------------------------'		
endif
		
ifdef ZTWA
;-------------------------------------------------------------------------
; LJ 9/6/2006 - Reset ZTAG, disable accelerator, use manual clock toggling
		call	ZTInit
;-------------------------------------------------------------------------
endif
		call	ZTPrepareRead
		
		call	ParmRecord2EAX		; fetch parameter record to EAX
		jnc	ParmOK
		
		SAYCRLF	'Flash programming start address missing!'
		jmp	ExitPgmrFail				
ParmOK:
		SAY	'Programming starts from 0x'		
		call	SerOut32
		call	CRLF
		
		and	eax, 0FFFFF000h
		mov	cs:[FlashBase], eax
		
		; Go on with basket now	
		
		PROCESS_BASKET_HEADER	cs:[BasketSize], ExitPgmrFail		; get header from basket and display information based on it
		
		; OK, bells and whistles are there. Do programming now.
		
		; Create Memory window 
		
		MEMWIN	CS0, _8BIT_RW, PRGBASE, (64*1024), cs:[FlashBase]
		
		; Turn ISA bus speed as slow as possible, since we do not know what the 
		; SYSCLK is and we can not program device well if we do it too fast
		
;--------------------------------------------------------------------
; Don't do this - LJ 8/21/2006		
;		SLOW_ISA DIV_8			; set maximum divider
;--------------------------------------------------------------------
						
		; Now we have chip layed out from E000:0 to E000:FFFF
		; Time to program some flash.
		
		; Check, whenever or not we have supported part on-board
		
		mov	al, 90h			; get ID
		call	JEDEC_Cmd_8
		
		push	(PRGBASE/10h)
		pop	ds		
		xor 	si, si
		mov	ax, ds:[si]		; get chip ID to EAX
		mov	bx, ax			; save ID
		
		SAY	'Device: Mfg='									
		call	SerOut8
		
		SAY	' DevID='
		shr	ax, 8
		call	SerOut8
		call	CRLF
		
		mov	al, 0F0h
		mov	ds:[si], al		; end identification mode		
		
		mov	di, offset Supported
CheckSupported:
		cmp	word ptr cs:[di], 0FFFFh	; end-of-table?
		jz	Unsupported
		
		cmp	bx, word ptr cs:[di]
		jz	ChipSupported
		add	di, 2+4+16+1
		jmp	CheckSupported
		
Unsupported:
		SAYCRLF	'Unsupported chip. Supported types are: '
		
		mov	di, offset Supported
		add	di, 2+4
		xor	cx, cx
ShowTypes:		
		call	SerSend
		call	CRLF
		add	di, 2+4+16+1
		cmp	cs:[di-6], 0FFFFh
		jz	@f		
		jmp	ShowTypes	
@@:
		call	CRLF		
		jmp	ExitPgmrFail		
		
ChipSupported:						
		SAY	'Detected Flash Device: '
		
		add	di, 2+4
		call	SerSend			; show chip ID
		call	CRLF
		
		; ====== Basket handling begins here ======
		;
		; Note, that in order to maintain correct checksum for basket, 
		; we must calculate data CRC for RLE header as well!
		;
		
		call	ResetCRC
		
		PROCESS_RLE_HEADER  ExitPgmrFail	; process RLE header if any
		
		mov	ebp, cs:[BasketSize]		; total bytes to program
PrgLoop:
		; We have right chip. Erase sector now
		
		SAY	'0x'		
		
		mov	eax, cs:[FlashBase]
		call	SerOut32
		
		SAY	': Erasing .. '		
		
		push	(PRGBASE/10h)
		pop	fs
		xor	si, si
		
		mov	al, 80h			; sector erase command part 1
		call	JEDEC_Cmd_8
		mov	al, 30h			; sector erase command part 2
		call	JEDEC_Cmd_8
		
		; Erase is now in progress. Check, when we are ready.
				
		call	DSBX2Var
		mov	ax, 182	
		mov	ds:[bx.CountDown], ax	; gives 10sec. timeout
		
		mov	dx, 218h
		mov	al, 05Eh ;ZT_SIG_OUT
		out	dx, al
		inc	dx
		in	al, dx
		and	al, 11110101b		; disable LED's		
		out	dx, al
		
		call	ZTPrepareRead		; ZFLogic back to track 
@@:
		cmp	word ptr ds:[bx.CountDown], 0
		jz	@f		
		cmp	dword ptr fs:[si], 0FFFFFFFFh
		jnz	@b			; loop until erase completes
@@:
		mov	word ptr ds:[bx.CountDown], 0	; reset timer, so we will not loose our blinking LED's
		cmp	dword ptr fs:[si], 0FFFFFFFFh
		jz	@f
		
		SAYCRLF	'FAILED!'
		
		jmp	ExitPgmrFail
@@:		

		; All set. Programming ...
		
		SAY	'Programming .. '
		
		mov	ecx, 64*1024
		cmp	ecx, ebp
		jbe	@f
		mov	ecx, ebp		; if less than 64K left, program it		
@@:
		cmp	ecx, 0
		jz	PgmDone			; don't do anything if basket size is 0		
		
		push	(PRGBASE/10h)
		pop	ds
		xor	si, si
		
BytePrgLoop:
		mov	al, 0A0h		; "program" command
		call	JEDEC_Cmd_8
		
		call	ZTRead_RLE
		
		mov	ds:[si], al		; write byte		
		mov	bl, al			; save byte for future compare		
		
		; on the extreme case we can have PCI backside clock 80Mhz. ISA
		; divider is 8, so we have 10M ISA bus clock. Programming cycle can be max.
		; 50us, so we need to wait here about 500 ISA cycles to kill that time.
		
		mov	di, 500
WaitWriting:
		cmp	ds:[si], bl
		jz	ByteOk
		in	al, 80h			; create one ISA cycle for delay
		dec	di		
		jnz	WaitWriting
		
		SAYCRLF	'FAILED!'
		
		jmp	ExitPgmrFail		
ByteOk:
		inc	si			; advance address to next
		loopd	BytePrgLoop	
PgmDone:	
		SAYCRLF	'OK!'
				
		; Anything left?
		
		sub	ebp, 64*1024
		jz	@f
		jc	@f
		
		; move to next page
				
		add	dword ptr cs:[FlashBase], 64*1024	; update page to next		
		
		MEMWIN_FLASHBASE CS0, cs:[FlashBase]
		
		jmp	PrgLoop
		
		; Now when programming is done, go check the data checksum if it was RLE image
		
		cmp	byte ptr cs:[BasketType], 0FFh
		jz	@f
		
		mov	eax, dword ptr cs:[RLE_CheckSum]
		cmp	eax, dword ptr cs:[RLE_Chk]		; compare against calculated checksum
		jz	@f
		
		SAY	'RLE basket checksum error '
		
		call	SerOut32
		mov	al, ' '
		call	SerSend2
		mov	eax, dword ptr cs:[RLE_Chk]
		call	SerOut32	
		call	CRLF
		
		jmp	ExitPgmrFail
@@:
		call	ZTPrepareRead
		
		; Get original checksum
		
		call	ZTRead
		shl	ax, 8
		call	ZTRead
		xchg	al, ah
		
		call	DSBX2Var		; get variables block to DS:BX
		cmp	word ptr ds:[bx.YModemCRChi_C], ax
		jz	CRCOK
	
		SAY	'Data CRC failure: '
		
		call	SerOut16
		mov	al, ' '
		call	SerSend2
		mov	ax, word ptr ds:[bx.YModemCRChi_C]
		call	SerOut16
		call	CRLF		
		jmp	ExitPgmrFail		
CRCOK:
		SAYCRLF	'Data CRC was OK!'
		
		jmp	ExitPgmrOk		
ExitPgmrFail:
		; Lit up RED LED and do not do anything else!
		
		mov	dx, 218h	
		mov	al, 05Eh 		;ZT_SIG_OUT
		out	dx, al
		inc	dx
		in	al, dx
		and	al, 11110101b
		or	al, ZT_LED_RED		;00001000b
		out	dx, al	
		
		SAYCRLF	'Freezing because of Programming failure ...'		
	
		jmp	$
						
ExitPgmrOk:
		; Lit up GREEN LED and continue with BUR		
		mov	dx, 218h
		mov	al, 05Eh 		;ZT_SIG_OUT
		out	dx, al
		inc	dx
		in	al, dx
		and	al, 11110101b
		or	al, ZT_LED_GREEN	;00000010b
		out	dx, al
ExitPgmr:
		call	CRLF
		
		; Set timer to maximum value. This is useful to prevent
		; BUR from blinking with LED's when loading next commands.
		; This way we maintain our RED or GREEN LED setting
		
		call	DSBX2Var
		mov	word ptr ds:[bx.CountDown], 0FFFFh
		
		; Always exit with ZFL registers prepared for accelerated read!	
			
		call	ZTPrepareRead
				
		retf				; resume with BUR
		
		
rcsid		db '$Id: am29fxxx.asm 1.4 2006/09/15 16:13:32 luther Exp $'
	
CHIP	MACRO	id, size_bytes, chipname, pad
	dw	id
	dd	size_bytes
	db	chipname
	db	0
	db	pad
ENDM
		
Supported	label word
		CHIP	0AD01h, (2048*1024), "AM29F016B", "       "	; OK. (artec)
		CHIP	0A401h,  (512*1024), "AM29F040B", "       "	; not tested
		CHIP	00401h,  (512*1024), "AM29F040B-2", "     "	; OK. (cmarez)
		CHIP	04101h, (4096*1024), "AM29F032B", "       "	; not tested
		CHIP	03D01h, (2048*1024), "AM29F017D", "       "	; not tested (backward compatible chips: F016C, F017B)
		CHIP	0D501h, (1024*1028), "AM29F080B", "       "	; not tested
		CHIP	04F01h,  (512*1024), "AM29LV040B", "      "	; not tested
		CHIP	03801h, (1024*1024), "AM29LV081B", "      "	; not tested
		
		dw	0FFFFh	; end-of-table


FlashBase	dd	?

		include BURAPI.ASM
		include FLASHPGM.ASM	; flash programmer common routines

END START
ENDS
